from __future__ import absolute_import

import datetime
import logging
from contextlib import contextmanager

from lola.utils.config import LolaEnv
from lola.utils.injection import injector

from lola.itinerary_parsing.utils import first
from redis import Redis

logger = logging.getLogger('lola.itinerary_parser')


class LolaService(object):
    @contextmanager
    def rate_limit(self, key, expires):
        config = injector.get(LolaEnv)
        redis = injector.get(Redis)

        if config.ENVIRONMENT == 'dev':
            yield False
        else:
            key = 'rate_limit:%s:%s' % (self.__class__.__name__.lower(), key)
            if redis.exists(key):
                yield True
            else:
                yield False
                redis.setex(key, True, expires - datetime.datetime.utcnow())

    def lookup(self, interface):
        if isinstance(interface, basestring):
            interface = first([b[0] for b in injector.binder._bindings if b[0].__name__ == interface])
        return injector.get(interface)

    def call(self, f, *args, **kwargs):
        return injector.get(f)(*args, **kwargs)
