"""
This module helps you to make an analysis of the json results
we get out of the parser, helps you to get missing fields,
values and invalid values in the json.
"""
import logging

logger = logging.getLogger('lola.model')

class EmailLogging(object):

   def __init__(self, json_schema, parsed_email):
        self.json_schema = json_schema
        self.parsed_email = parsed_email

   def validate_email(self):
       missing_values, missing_field, invalid_values, total_key = self.\
           get_missing_val_fields(self.json_schema, self.parsed_email)
       return missing_values, missing_field, invalid_values, EmailLogging.\
           get_percentage_completed(missing_values + missing_field + invalid_values, total_key)


   @staticmethod
   def get_percentage_completed(missed_fields, total_keys):
        """
        Calculates completed percentage of parsed email according to validation schema
        """
        if total_keys > 0:
            return (float(total_keys - len(missed_fields)) / total_keys) * 100
        return 0


   def get_missing_val_fields(self, json_schema, parsed_email):
       try:
            missing_val = []
            missing_field = []
            invalid_values = []
            json_key_count = 0

            for key, val in json_schema.iteritems():
                json_key_count += 1
                if key not in parsed_email:
                    missing_field.append(key)
                    continue

                key_value = parsed_email.get(key)
                if not key_value:
                    missing_val.append(key)
                elif isinstance(val, dict):
                   json_key_count += self.parse_child(missing_val, missing_field, invalid_values, json_key_count,
                                     json_schema.get(key), key_value)
                elif isinstance(val, list):
                    if key_value:
                        for index, item in enumerate(val):
                            try:
                              json_key_count += self.parse_child(missing_val, missing_field, invalid_values, json_key_count,
                                                 item,  key_value[index])
                            except IndexError, error:
                                logger.log('Error while parsing list in with a dict: %s', error.message)
                                break
                    else:
                        missing_val.append(key)
                else:
                    EmailLogging.get_invalid_fields(parsed_email.get(key), key, invalid_values,
                                            json_schema.get(key))

            return missing_val, missing_field, invalid_values, json_key_count
       except Exception, e:
            logger.exception('Error : %s', e.message)

   def parse_child(self, missing_val, missing_field, invalid_values, json_key_count, json_schema,
                   parsed_email):
       val1, val2, val3, json_key = self.get_missing_val_fields(json_schema, parsed_email)
       missing_val.extend(val1)
       missing_field.extend(val2)
       invalid_values.extend(val3)
       return json_key

   @staticmethod
   def get_invalid_fields(val, key, invalid_values, val_type):
       """
       Validate type of value according to type, mentioned in validation schema
       """
       try:
        val_type(val)
       except ValueError, error:
        invalid_values.append(key)
