# -*- coding: UTF-8 -*-
"""
The file contains the parsing/overrides functions for Air Canada airline emails.
"""

from __future__ import absolute_import
import logging
from lola.itinerary_parsing.utils import overrides

from lola.itinerary_parsing.parsers.email_parsers import travel_filter
from lola.itinerary_parsing.parsers.email_parsers.airlines import AirlineFilter
from lola.itinerary_parsing.parsers.email_parsers.rules.airlines.aircanada import AIRCANADA_RULES, \
    AIRCANADA_RULES_ORDER, DATE_FORMAT, SEGMENT_IDENTIFY_RULES

logger = logging.getLogger('lola.airline')


@travel_filter(source='Air Canada Airlines', sender='leandra@lolatravel.com', subject=['Air Canada'])
@travel_filter(source='Air Canada Airlines', sender='confirmation@aircanada.ca', subject=['Air Canada'])
# pylint: disable=no-self-use
class AirCanadaFilter(AirlineFilter):
    """
    Base class for Air Canada airline emails parsing.
    Override functions for Air Canada airline are in this class
    """

    def __init__(self):
        super(AirCanadaFilter, self).__init__()
        self.airline_rules = AIRCANADA_RULES
        self.airline_rules_order = AIRCANADA_RULES_ORDER
        self.date_format = DATE_FORMAT
        self.segment_identify_rule = SEGMENT_IDENTIFY_RULES

    @overrides(AirlineFilter)
    def filter_source(self):
        """
        Return the source name of airline and currently
         'Air Canada Airlines' is returning
        """
        return 'Air Canada Airlines'
