# -*- coding: UTF-8 -*-
"""
The file contains the parsing/overrides functions for Air France airline emails.
"""

from __future__ import absolute_import
import logging

from lola.itinerary_parsing.utils import overrides
from lola.itinerary_parsing.parsers.email_parsers import travel_filter
from lola.itinerary_parsing.parsers.email_parsers.airlines import AirlineFilter
from lola.itinerary_parsing.parsers.email_parsers.rules.airlines.airfrance import AIR_FRANCE_RULES, \
    AIR_FRANCE_RULE_ORDER, DATE_FORMAT, SEGMENT_IDENTIFY_RULES

logger = logging.getLogger('lola.airline')


@travel_filter(source='Air France Airlines', sender='leandra@lolatravel.com',
               subject=['Confirmation of your Air France booking'])
@travel_filter(source='Air France Airlines', sender='info@service.airfrance.com',
               subject=['Confirmation of your Air France booking'])
# pylint: disable=no-self-use
class AirFranceFilter(AirlineFilter):
    """
    Base class for Air France airline emails parsing.
    Override functions for Air France airline are in this class
    """

    def __init__(self):
        super(AirFranceFilter, self).__init__()
        self.airline_rules = AIR_FRANCE_RULES
        self.airline_rules_order = AIR_FRANCE_RULE_ORDER
        self.date_format = DATE_FORMAT
        self.segment_identify_rule = SEGMENT_IDENTIFY_RULES

    @overrides(AirlineFilter)
    def filter_source(self):
        """
        Return the source name of airline and currently
         'Air France Airlines' is returning
        """
        return 'Air France Airlines'

    @overrides(AirlineFilter)
    def get_charge_category_list(self):
        """
        Function to return charges category list
        """
        # There are purchase values which we don't want to include into our result
        # so exclude category will determine such values
        return ['exclude', 'total', 'base', 'tax', 'fees']
