"""
The file contains the parsing/overrides functions for Alaska Airlines emails.
"""
# -*- coding: UTF-8 -*-
from __future__ import absolute_import
import logging

from lola.itinerary_parsing.utils import overrides
from lola.itinerary_parsing.parsers.email_parsers import travel_filter
from lola.itinerary_parsing.parsers.email_parsers.airlines import AirlineFilter
from lola.itinerary_parsing.parsers.email_parsers.rules.airlines.alaska import ALASKA_RULES, \
    ALASKA_RULE_ORDER, DATE_FORMAT, SEGMENT_IDENTIFY_RULES, BACKWARD_TRAVERSAL_LIMIT, FORWARD_TRAVERSAL_LIMIT

logger = logging.getLogger('lola.airline')


@travel_filter(source='Alaska Airlines', sender='rachel.neasham@gmail.com', subject=['Confirmation Letter'])
@travel_filter(source='Alaska Airlines', sender='alaska.it@alaskaair.com', subject=['Confirmation Letter'])
@travel_filter(source='Alaska Airlines', sender='leandra@lolatravel.com', subject=['Confirmation Letter'])
# pylint: disable=no-self-use
class AlaskaFilter(AirlineFilter):
    """
     Base class for Alaska Airlines emails parsing.
    Override functions for Alaska Airlines are in this class
    """

    def __init__(self):
        super(AlaskaFilter, self).__init__()
        self.airline_rules = ALASKA_RULES
        self.airline_rules_order = ALASKA_RULE_ORDER
        self.date_format = DATE_FORMAT
        self.segment_identify_rule = SEGMENT_IDENTIFY_RULES
        self.back_traversal_limit = BACKWARD_TRAVERSAL_LIMIT
        self.forward_traversal_limit = FORWARD_TRAVERSAL_LIMIT

    @overrides(AirlineFilter)
    def filter_source(self):
        """
        Return the source name of airline and currently
        'Alaska Airlines' is returning
        """
        return 'Alaska Airlines'

    @overrides(AirlineFilter)
    def get_charge_category_list(self):
        """
        Function to return charges category list
        """
        # There are purchase values which we don't want to include into our result
        # so exclude category will determine such values
        return ['exclude', 'total', 'base', 'tax', 'fees']
