# -*- coding: UTF-8 -*-
"""
The file contains the parsing/overrides functions for American airline emails.
"""

from __future__ import absolute_import
import logging

from lola.itinerary_parsing.utils import overrides
from lola.itinerary_parsing.parsers.email_parsers import travel_filter
from lola.itinerary_parsing.parsers.email_parsers.airlines import AirlineFilter
from lola.itinerary_parsing.parsers.email_parsers.rules.airlines.american import AMERICAN_RULE, \
    AMERICAN_RULES_ORDER, DATE_FORMAT, SEGMENT_IDENTIFY_RULES

logger = logging.getLogger('lola.airline')

@travel_filter(source='American Airlines', sender='rg.vanessa@gmail.com', subject=['E-Ticket Confirmation'])
@travel_filter(source='American Airlines', sender='omair.shamshir@arbisoft.com', subject=['E-Ticket Confirmation'])
@travel_filter(source='American Airlines', sender='americanairlines@aa.com', subject=['E-Ticket Confirmation'])
@travel_filter(source='American Airlines', sender='americanairlines@checkin.email.aa.com',
               subject=['E-Ticket Confirmation'])
# pylint: disable=no-self-use
class AmericanFilter(AirlineFilter):
    """
    Base class for American airline emails parsing.
    Override functions for American airline are in this class
    """

    def __init__(self):
        super(AmericanFilter, self).__init__()
        self.airline_rules = AMERICAN_RULE
        self.airline_rules_order = AMERICAN_RULES_ORDER
        self.date_format = DATE_FORMAT
        self.segment_identify_rule = SEGMENT_IDENTIFY_RULES

    @overrides(AirlineFilter)
    def filter_source(self):
        """
        Return the source name of airline and currently 'American Airlines' is returning
        """
        return 'American Airlines'

    @overrides(AirlineFilter)
    def parser_class(self):
        """
        Return class of the parser, currently this returns 'html.parser'
        """
        return 'html.parser'

    @overrides(AirlineFilter)
    def sum_purchase_amounts(self, charges_keys_to_values_dict):
        purchase_charges = {}
        total_sum = 0
        try:
            for category, charges_info in charges_keys_to_values_dict.iteritems():
                for _, charges in charges_info.iteritems():
                    total_sum += sum(charges)
                purchase_charges[category] = total_sum
                total_sum = 0
        except TypeError:
            pass
        return purchase_charges
