"""
The file contains the parsing/overrides functions for Frontier airline emails.
"""
from __future__ import absolute_import
import logging

from lola.itinerary_parsing.utils import overrides
from lola.itinerary_parsing.parsers.email_parsers import travel_filter
from lola.itinerary_parsing.parsers.email_parsers.airlines import AirlineFilter
from lola.itinerary_parsing.parsers.email_parsers.rules.airlines.frontier import FRONTIER_RULES, \
    FRONTIER_RULE_ORDER, DATE_FORMAT, SEGMENT_IDENTIFY_RULES

logger = logging.getLogger('lola.airline')


@travel_filter(source='Frontier Airlines', sender='no-reply@flyfrontier.com', subject=['Reservation Confirmation'])
@travel_filter(source='Frontier Airlines', sender='leandra@lolatravel.com', subject=['Reservation Confirmation'])
# pylint: disable=no-self-use
class FrontierFilter(AirlineFilter):
    """
    Base class for Frontier airline emails parsing.

    Override functions for Frontier airline are in this class
    """

    def __init__(self):
        super(FrontierFilter, self).__init__()
        self.airline_rules = FRONTIER_RULES
        self.airline_rules_order = FRONTIER_RULE_ORDER
        self.date_format = DATE_FORMAT
        self.segment_identify_rule = SEGMENT_IDENTIFY_RULES

    @overrides(AirlineFilter)
    def filter_source(self):
        """
        Return the source name of airline and
         currently 'Frontier Airlines' is returning
        """
        return 'Frontier Airlines'
