"""
The file contains the parsing/overrides functions for KLM airline emails.
"""
from __future__ import absolute_import
import logging

from lola.itinerary_parsing.utils import overrides
from lola.itinerary_parsing.parsers.email_parsers import travel_filter
from lola.itinerary_parsing.parsers.email_parsers.airlines import AirlineFilter
from lola.itinerary_parsing.parsers.email_parsers.rules.airlines.klm import KLM_AIRLINE_SUBJECT_REGEX, \
    KLM_RULES, KLM_RULE_ORDER, DATE_FORMAT, SEGMENT_IDENTIFY_RULES

logger = logging.getLogger('lola.airline')


@travel_filter(source='KLM Airlines', sender='leandra@lolatravel.com', subject=None, regex=[KLM_AIRLINE_SUBJECT_REGEX])
@travel_filter(source='KLM Airlines', sender='reservations.usa@klm.com', subject=None,
               regex=[KLM_AIRLINE_SUBJECT_REGEX])
# pylint: disable=no-self-use
class KlmFilter(AirlineFilter):
    """
    Base class for KLM airline emails parsing.
    Override functions for KLM airline are in this class
    """

    def __init__(self):
        super(KlmFilter, self).__init__()
        self.airline_rules = KLM_RULES
        self.airline_rules_order = KLM_RULE_ORDER
        self.date_format = DATE_FORMAT
        self.segment_identify_rule = SEGMENT_IDENTIFY_RULES

    @overrides(AirlineFilter)
    def filter_source(self):
        """
        Return the source name of airline and currently
         'KLM Airlines' is returning
        """
        return 'KLM Airlines'

    @overrides(AirlineFilter)
    def parser_class(self):
        """
        Return class of the parser, currently this returns 'html.parser'
        """
        return "html.parser"
