"""
The file contains the parsing/overrides functions for Lufthansa Airlines emails.
"""
# -*- coding: UTF-8 -*-
from __future__ import absolute_import
import logging
import copy

from lola.itinerary_parsing.utils import overrides
from lola.itinerary_parsing.parsers.email_parsers import travel_filter
from lola.itinerary_parsing.parsers.email_parsers.airlines import AirlineFilter
from lola.itinerary_parsing.parsers.email_parsers.rules.airlines.lufthansa import LUFTHANSA_RULE, \
    LUFTHANSA_RULE_ORDER, DATE_FORMAT, SEGMENT_IDENTIFY_RULES, FORWARD_TRAVERSAL_LIMIT

logger = logging.getLogger('lola.airline')

@travel_filter(source='Lufthansa Airlines', sender='online@booking-lufthansa.com', subject=['Booking Details'])
@travel_filter(source='Lufthansa Airlines', sender='leandra@lolatravel.com', subject=['Booking Details'])
@travel_filter(source='Lufthansa Airlines', sender='travel-emails@blade.net', subject=['Booking Details'])
# pylint: disable=no-self-use
class LufthansaFilter(AirlineFilter):
    """
     Base class for Lufthansa Airlines emails parsing.
    Override functions for Lufthansa Airlines are in this class
    """

    def __init__(self):
        super(LufthansaFilter, self).__init__()
        self.airline_rules = LUFTHANSA_RULE
        self.airline_rules_order = LUFTHANSA_RULE_ORDER
        self.date_format = DATE_FORMAT
        self.segment_identify_rule = SEGMENT_IDENTIFY_RULES
        self.forward_traversal_limit = FORWARD_TRAVERSAL_LIMIT

    @overrides(AirlineFilter)
    def filter_source(self):
        """
        Return the source name of airline and currently
        'Spirit Airlines' is returning
        """
        return 'Lufthansa Airlines'

    @overrides(AirlineFilter)
    def get_charges_pattern(self):
        """
        Function to return charges pattern dic
        """
        # todo: custom pattern should handle in rules file instead of code
        flight_charges_pattern = copy.deepcopy(self.charge_pattern)
        flight_charges_pattern['base'].append('price')
        return flight_charges_pattern
