"""
The file contains the parsing/overrides functions for Spirit Airlines emails.
"""
# -*- coding: UTF-8 -*-
from __future__ import absolute_import
import logging

from lola.itinerary_parsing.utils import overrides
from lola.itinerary_parsing.parsers.email_parsers import travel_filter
from lola.itinerary_parsing.parsers.email_parsers.airlines import AirlineFilter
from lola.itinerary_parsing.parsers.email_parsers.rules.airlines.spirit import SPIRIT_RULES, SPIRIT_RULE_ORDER, \
    DATE_FORMAT, SEGMENT_IDENTIFY_RULES, FORWARD_TRAVERSAL_LIMIT

logger = logging.getLogger('lola.airline')


@travel_filter(source='Spirit Airlines', sender='booking@t.spiritairlines.com',
               subject=['Spirit Airlines Flight Confirmation'])
@travel_filter(source='Spirit Airlines', sender='leandra@lolatravel.com',
               subject=['Spirit Airlines Flight Confirmation'])
# pylint: disable=no-self-use
class SpiritFilter(AirlineFilter):
    """
     Base class for Spirit Airlines emails parsing.
    Override functions for Spirit Airlines are in this class
    """

    def __init__(self):
        super(SpiritFilter, self).__init__()
        self.airline_rules = SPIRIT_RULES
        self.airline_rules_order = SPIRIT_RULE_ORDER
        self.date_format = DATE_FORMAT
        self.segment_identify_rule = SEGMENT_IDENTIFY_RULES
        self.forward_traversal_limit = FORWARD_TRAVERSAL_LIMIT

    @overrides(AirlineFilter)
    def filter_source(self):
        """
        Return the source name of airline and currently
        'Spirit Airlines' is returning
        """
        return 'Spirit Airlines'
