"""
The file contains the parsing/overrides functions for United airline emails.
"""
from __future__ import absolute_import
import logging

from lola.itinerary_parsing.utils import overrides
from lola.itinerary_parsing.parsers.email_parsers import travel_filter
from lola.itinerary_parsing.parsers.email_parsers.airlines import AirlineFilter
from lola.itinerary_parsing.parsers.email_parsers.rules.airlines.united import UNITED_RULE, UNITED_RULES_ORDER,\
    DATE_FORMAT, SEGMENT_IDENTIFY_RULES, DATE_PATTERN2

logger = logging.getLogger('lola.airline')


@travel_filter(source='United Airlines', sender='unitedairlines@united.com',
               subject=['eTicket Itinerary and Receipt for Confirmation', 'united.com reservation for'])
@travel_filter(source='United Airlines', sender='ankitdesai88@gmail.com',
               subject=['eTicket Itinerary and Receipt for Confirmation', 'united.com reservation for'])
# pylint: disable=no-self-use
class UnitedFilter(AirlineFilter):
    """
    Base class for United airline emails parsing.
    Override functions for United airline are in this class
    """

    def __init__(self):
        super(UnitedFilter, self).__init__()
        self.airline_rules = UNITED_RULE
        self.airline_rules_order = UNITED_RULES_ORDER
        self.date_format = DATE_FORMAT
        self.segment_identify_rule = SEGMENT_IDENTIFY_RULES

    @overrides(AirlineFilter)
    def filter_source(self):
        """
        Return the source name of airline and currently 'United Airlines' is returning
        """
        return 'United Airlines'

    @overrides(AirlineFilter)
    def parser_class(self):
        """
        Return class of the parser, currently this returns 'html.parser'
        """
        return 'html.parser'

    @overrides(AirlineFilter)
    def enhance_flight_info(self, segments, flight_info, value, soup):
        if flight_info != 'flight_date':
            return
        if 'flight_date' in segments and segments.get('flight_date'):
            return

        result = soup.find(text=DATE_PATTERN2)
        if result:
            segments.update({flight_info: [str(result)]})
