from __future__ import absolute_import

import re
from lola.itinerary_parsing.utils import overrides
from bs4 import NavigableString

from lola.itinerary_parsing.parsers.email_parsers.hotels import HotelFilter
from lola.itinerary_parsing.parsers.email_parsers import travel_filter
from lola.itinerary_parsing.utils import strip_unicode, remove_empty_childs


@travel_filter(source='ChoiceHotels', sender='leandra@lolatravel.com',
               subject=['Fwd: Reservation Confirmation for Tuesday, Mar 22, 2016'])
@travel_filter(source='ChoiceHotels', sender='yourstay@choicehotels.com',
               subject=['Reservation Confirmation'])
class ChoiceHotelsFilter(HotelFilter):
    """
    This class contains methods to extract info
    out of the email which couldn't be extracted by base class
    method
    """

    @overrides(HotelFilter)
    def filter_source(self):
        return 'ChoiceHotels'

    @overrides(HotelFilter)
    def extract_hotel_description(self, soup, hotel_name):
        """
        Try to extract hotel description out of the email.
        This function tries to create RE from hotel name and
        then finds anchor tag against matched text, since this
        is an anchor whose previous sibling holds hotel description.
        """
        hotel_description = None
        if hotel_name:
            hotel_name = r'\s*)+('.join((hotel_name.split()))
            hotel_name = '(%s)?' % hotel_name

            anchor = soup.find('a', text=re.compile(hotel_name, re.IGNORECASE))
            if anchor:
                remove_empty_childs(anchor.parent)
                if anchor.previous_sibling:
                    anchor = anchor.previous_sibling
                    if isinstance(anchor, NavigableString):
                        hotel_description = anchor
                    else:
                        hotel_description = anchor.text
        return str(strip_unicode(hotel_description)) if hotel_description else None
