# -*- coding: UTF-8 -*-

"""
Emails for hilton hotels have structure that is very
different from the other emails, so lot of functionality
has to be over-written. We will have to update the
base parser to handle such cases
"""

from __future__ import absolute_import

import re
from lola.itinerary_parsing.utils import overrides
from bs4 import NavigableString

from lola.itinerary_parsing.parsers.email_parsers.rules.hotels import TOKENS_ADDRESS
from lola.itinerary_parsing.parsers.email_parsers.hotels import HotelFilter
from lola.itinerary_parsing.parsers.email_parsers import travel_filter

from lola.itinerary_parsing.parsers.email_parsers.rules.hotels.hilton import PATTERN_SUBJECT
from lola.itinerary_parsing.utils import remove_empty_childs


@travel_filter(source='Hilton', sender='@hilton.com', subject=['sent you an email from'])
@travel_filter(source='Hilton', sender='@res.hilton.com', subject=['Confirmation'])
@travel_filter(source='Hilton', sender='leandra@lolatravel.com', subject=['Your 18 Feb 2016 Confirmation'])
@travel_filter(source='Hilton', sender='christianbusch@gmail.com', subject=['Your Sep 11, 2015 Confirmation'])
@travel_filter(source='Hilton', sender='hiltonhotels&resorts@res.hilton.com', subject=['Confirmation'])
@travel_filter(source='Hilton', sender='doubletreebyhilton@res.hilton.com',
               subject=['DoubleTree by Hilton Confirmation'])
@travel_filter(source='Hilton', sender='omair.shamshir@arbisoft.com', subject=['Homewood Confirmation'])
@travel_filter(source='Hilton', sender='omair.shamshir@arbisoft.com', subject=['Hampton Confirmation'])
@travel_filter(source='Hilton', sender='omair.shamshir@arbisoft.com', subject=['Embassy Suites Confirmation'])
@travel_filter(source='Hilton', sender='doubletreebyhilton@res.hilton.com', subject=['Confirmation'])
class HiltonFilter(HotelFilter):
    """
    This class contains methods to extract info
    out of the email which couldn't be extracted by base class
    method
    """

    def __init__(self):
        super(HiltonFilter, self).__init__()
        self.multiline = True

    @overrides(HotelFilter)
    def filter_source(self):
        return 'Hilton'

    @overrides(HotelFilter)
    def extract_hotel_name(self, email, soup):
        """
        Extract hotel name from the email. Structure of hilton is
        different from the other hotel emails. So this is to be handled
        differently
        """

        def collect_sib_text(node):
            remove_empty_childs(node.parent)
            # Purpose of adding TD in check is that we are assuming name for the hotel
            # is suppose to be present above address and telephone number rather than
            # behind it

            # TODO: Looking for better approach to handle this since picking value of element being
            # dependant on specific element 'td' looks a bit awkward
            if not node.previous_sibling or node.parent.name == 'td':
                return collect_sib_text(node.parent)

            try:
                name_node = node.parent.contents[0]
                if isinstance(name_node, NavigableString):
                    return name_node.strip()
                return name_node.getText('').strip()
            except (AttributeError, IndexError):
                return None

        node = soup.find(text=re.compile(r'(%s)' % ('|'.join(TOKENS_ADDRESS))))
        if node:
            hotel_name = collect_sib_text(node)
            if hotel_name:
                return hotel_name
        if not hotel_name:
            return super(HiltonFilter, self).extract_hotel_name(email, soup)

    @overrides(HotelFilter)
    def parse_record_locator(self, email, soup):
        match = PATTERN_SUBJECT.match(email.subject)
        if match:
            return match.group(1).strip()
        return super(HiltonFilter, self).parse_record_locator(email, soup)

    # TODO: Not sure whether hilton have any other reward program other then Hilton HHonors.
    # If any then we need to add code to extract it on runtime
    def extract_reward_program(self, email, soup):
        return 'Hilton HHonors'
