"""
We have got a specific case in which
there was no way of extracting
Arrival/Departure time generically
"""
from __future__ import absolute_import

from lola.itinerary_parsing.utils import overrides
from lola.itinerary_parsing.parsers.email_parsers.hotels import HotelFilter
from lola.itinerary_parsing.parsers.email_parsers import travel_filter
from lola.itinerary_parsing.parsers.email_parsers.rules.hotels import PATTERN_ARRIVAL_DEPARTURE_TIME

from lola.itinerary_parsing.parsers.email_parsers.rules.hotels.ihg import PATTERN_RECORD_LOCATOR


@travel_filter(source='InterContentinental', sender='leandra@lolatravel.com',
               subject=['Your Reservation Confirmation'])
@travel_filter(source='InterContentinental', sender='CandlewoodSuites@reservations.ihg.com',
               subject=['Your Reservation Confirmation'])
class IHGFilter(HotelFilter):
    """
    This class contains methods to extract info
    out of the email which couldn't be extracted by base class
    method
    """

    @overrides(HotelFilter)
    def filter_source(self):
        return 'InterContentinental'

    @overrides(HotelFilter)
    def parse_arrival_departure_time(self, soup, matched_pattern, multiline=False):
        """
        Try to pull arrival and departure time out of email text
        """
        arrival_time = None
        departure_time = None
        if not arrival_time and not departure_time:
            results = soup.findAll(text=PATTERN_ARRIVAL_DEPARTURE_TIME)
            if results and len(results) > 1:
                arrival_time = results[0]
                departure_time = results[1]
        return arrival_time, departure_time

    @overrides(HotelFilter)
    def parse_record_locator(self, email, soup):
        match = PATTERN_RECORD_LOCATOR.search(email.filtered_subject)
        if match:
            return match.group(1).strip()
        return None
