"""
This file contains regular expression  that are using in Alaska airlines filter class
"""
import re
# To get specific email according to regex while scanning emails
# eg:Confirmation Letter - VZKBAT 03/13/16 - from Alaska Airlines
TRAVEL_FILTER_REGEX = r'(Letter\s+-\s*[A-Z]+\s+\d+\/\d+\/\d+\s+-\s+from\sAlaska)'

# To get flight operator and flight number with regex
FLIGHT_NUMBER_REGEX = re.compile(r"""
   (?xi)            # verbose regex,  case-insensitive
    (?:             # Non-capturing
        flight:\s*  # match a string 'Flight' follow by white space and character :
     )
    [A-z ]+     # match character in range A to Z and a to z or match any white space character
    \s*    # whitespace character
    (\d+)  # match one or more digits
    """)

FLIGHT_NUMBER_REGEX2 = re.compile(r"""
   (?x)
   (?:[A-z \-\\,]+)\s+(\d+)\s*\b
    """)

# To get flight operator and flight number with regex
OPERATING_AIR_LINE = re.compile(r"""
   (?xi)            # verbose regex,  case-insensitive
    (?:             # Non-capturing
        flight:\s*  # match a string 'Flight' follow by white space and character :
     )
    ([A-z ]+)     # match character in range A to Z and a to z or match any white space character
    \s*    # whitespace character
    \d+  # match one or more digits
    """)

OPERATING_AIR_LINE2 = re.compile(r"""
   (?x)
   ([A-z \-\\,]+)\s+(?:\d+)\s*\b
    """)

# To get flight plan type with regex eg: Equipment: Bombardier Q400
PLANE_TYPE_REGEX = re.compile(r"""
    (?xi)             # verbose regex,  case-insensitive
    (?:               # Non-capturing
        equipment:\s* # match a string 'equipment' follow by character : and one or unlimited white space character
     )
    (.*)              # matches any character (except newline)
    |
    ([a-z,]+\s*[a-z,]*\s*\d+\-\d+)
    """)

# To get all flight class with regex eg : Class: R(Coach)
SEAT_CLASS_REGEX = re.compile(r"""
    (?x)    # verbose regex,  case-insensitive
    (?:     # Non-capturing
        Class:\s* # match a string 'Class' follow by character :
     )
    (.*)    # matches any character (except newline)
    |
    ([A-Z]\s*\(\s*[A-z]+\s*\))
    """)

BACKWARD_TRAVERSAL_LIMIT = 4
FORWARD_TRAVERSAL_LIMIT = 6
DATE_FORMAT = ['%a, %b %d']


def is_more_expansion_needed_segment(flight_info, is_single):
    airport = flight_info.get('airport_code')
    return not (airport and len(airport) > 1)


AIRPORT_CODE = re.compile(r"""
                                (?x) # Example: San Francisco (SFO)
                                (?:
                                    [A-z \,\-\\]+
                                )
                                \s*
                                \(
                                    ([A-Z]+)
                                \)
                            """)

AIRPORT_CITY = re.compile(r"""
                                (?x) # Example: San Francisco (SFO)
                                (
                                    [A-z \,\-\\]+
                                    \s*
                                    \(
                                        [A-Z]+
                                    \)
                                )
                            """)

DATE_PATTERN = re.compile(r"""
                            (?x) # Example: Thu, Oct 10
                            (
                                [A-z]{3},\ +[A-z]{3}\ + \d{1,2}
                            )
                        """)

PASSENGER_NAME_REGEX = re.compile(r"""
                                        (?x) # Example: Blade Leanne
                                            (
                                                [A-Z][a-z\\\-]{3,}\s+[A-Z][a-z\\\-]{3,}
                                            )
                                        """)

PASSENGER_NAME_REGEX2 = re.compile(r"""
                                        (?x) # Example: Blade Leanne
                                            (
                                                [A-Z][a-z\\\-]{3,}\s+[A-Z][a-z\\\-]{3,}
                                            )
                                            \s*
                                            (?:
                                                \d{1,2}[A-Z]
                                            )
                                        """)

TIME_PATTERN = re.compile(r"""
                               (?xi) # Example: 10:15 AM
                                (?:
                                    [^:]
                                )
                                \s*
                                (
                                    \s+\d{1,2}\s*:\s*\d{1,2}\s*[ap].?[m]
                                )
                                """)

ALASKA_RULES = {
    'airport_code': {'rule': AIRPORT_CODE, 'is_repeated': True, 'is_replace': False},
    'airport': {'rule': AIRPORT_CITY, 'is_repeated': True, 'is_replace': True},
    'flight_number': [{'rule': FLIGHT_NUMBER_REGEX, 'is_repeated': False, 'is_replace': True},
                      {'rule': FLIGHT_NUMBER_REGEX2, 'is_repeated': False, 'is_replace': True}],
    'operating_airline': [{'rule': OPERATING_AIR_LINE, 'is_repeated': False, 'is_replace': False},
                          {'rule': OPERATING_AIR_LINE2, 'is_repeated': False, 'is_replace': False}],
    'plane_type': {'rule': PLANE_TYPE_REGEX, 'is_repeated': False, 'is_replace': True},
    'flight_date': {'rule': DATE_PATTERN, 'is_repeated': True, 'is_replace': True},
    'time': {'rule': TIME_PATTERN, 'is_repeated': True, 'is_replace': True},
    'seat_class': {'rule': SEAT_CLASS_REGEX, 'is_repeated': False, 'is_replace': True},
    'name': {'rule': PASSENGER_NAME_REGEX2, 'is_repeated': False, 'is_replace': True},
    'is_expansion_needed': is_more_expansion_needed_segment
}

PASSENGERS_RULE = {
    'name': {'rule': PASSENGER_NAME_REGEX, 'is_repeated': True, 'is_replace': True}
}

PASSENGER_SEGMENT_IDENTIFIER = re.compile(r"""^(?xi)\s*travelers?\s*$""")

SEGMENT_IDENTIFY_RULES = [[{'rule': AIRPORT_CODE, 'segment_rule': ALASKA_RULES, 'is_validate': False}
                           ], [{'rule': PASSENGER_SEGMENT_IDENTIFIER, 'segment_rule': PASSENGERS_RULE,
                                'is_validate': False}, ]]

# It is important to maintain rules order for the airlines.
# It is an extra step to make sure rules must not conflict with any other information of the flight.
# When any of the info matches to rule than we replace matched text with an empty string
# to be on safe side for the next search.
ALASKA_RULE_ORDER = ['name', 'airport_code', 'airport', 'flight_date', 'time',
                     'seat_class',
                     'plane_type', 'operating_airline', 'flight_number']
