"""
This file contains regular expression  that are using in american airlines filter class
"""
import re
from lola.itinerary_parsing.parsers.email_parsers.rules.airlines import TIME_PATTERN

# To get rewards program number (FF means Frequent flier) eg : FF#: URJ9242
FF_REGEX = re.compile(r"""
   (?x) # verbose regex
   FF\#:    # match characters FF follow by special character :
   \s+      # match one or more white space character
   (.*)     # match anything except new line
    """)

# To get Flight seat class name/category eg : economy,business
FLIGHT_CLASS_REGEX = re.compile(r"""
   (?xi) # verbose regex, case-insensitive
   (\b  #start word boundary
        economy|    # match word Economy
        business|   # or match word BUSINESS
        first\s*cl| # or match word First Cl
        discount|   # or match word Discount
        coach|      # or match word Coach
        first       # or match word First
    \b)
      """)

# To get flight operator name with regex eg: OPERATED BY
FLIGHT_OPERATOR_REGEX = re.compile(r"""
   (?x)   # verbose regex
    (?:   # non capturinga
        OPERATED\sBY\s)   # match string 'OPERATED BY' follow by white space
    ([A-Z|\s]*)     # match zero or more character from range A to Z or white space
    """)

DATE_FORMAT = ['%a %d%b']

# Regex for match date pattern like 'SUN 01FEB'
DATE_PATTERN = re.compile(r"""
    (?x)        # verbose regex
    [A-Z]{3}    # match character from range [A-Z] and repeat three times
    \s          # match a white space character
    \d{2}       # match digit and repeats two times
    [A-Z]{3}    # match character from range [A-Z] and repeat three times
    """)

# To match airport names like 'CHICAGO OHARE'
AIRPORT_NAME = re.compile(r"""
    (?xs)   # verbose regex, dotall regex
    (
        (?:[A-Z]{2,})\s   # match character from range [A-Z] and repeat more than two times and follow by a white space character
        (?:[A-Z]{2,})?    # match character from range [A-Z] and repeat more than two times and that is optional
        \ +(?:[A-Z]{2,})? # match space more than one  and  character from range [A-Z] and repeat more than
    )                     # two times and that is optional
    (?![a-z]+)            # Negative Lookahead - Assert that it is impossible to match the regex '[a-z]+'
    """)

# To match flight operating airline names eg : 'American 1065 '
FLIGHT_OPERATOR_PATTERN = re.compile(r"""
    (?x)                # verbose regex
    ^\s*                # assert position at start of the string follow by zero or more white space character
    ([A-z\s]+)          # match one or more characters from range [A-z] with white space
    (?:\s*\d{3,4}\s)    # Non-capturing group
    """)

# Regex to match passenger names pattern from string like 'JHON DAVID Seat 21E' or 'JHON DAVID FF#'
PASSENGER_NAME_PATTERN = re.compile(r"""
    (?xi)   # verbose regex, case-insensitive
    ^\s*    # assert position at start of the string follow by zero or more white space character
    (
        [a-z]+\s    # match one or more characters from range [a-z] follow by white space
        [a-z]*      # match zero or more characters from range [a-z]
    )
    \s*             # match zero or more white space character
    (?:seat\s*\d{1,2}[a-z]|FF\#)    # Non-capturing group (match word seat follow by white space and one to two digits
                                    # and character from [a-z])
    """)

# To match flight number from string eg : 'American 1441'
FLIGHT_NUMBER_REGEX = re.compile(r"""
    (?x)        # verbose regex
    (\d{3,4})   # match digit and repeat 3 to 4 times
    """)

# To match flight reward program name like 'Ff'
REWARD_PROGRAM = re.compile(r"""
    (?x)      # verbose regex
    (\s*FF)   # match zero or more white space character follow by word 'FF'
    """)

AMERICAN_RULE = {'name': {'rule': PASSENGER_NAME_PATTERN, 'is_repeated': True, 'is_replace': True},
                 'operating_airline': [{'rule': FLIGHT_OPERATOR_REGEX, 'is_repeated': True, 'is_replace': True},
                                       {'rule': FLIGHT_OPERATOR_PATTERN, 'is_repeated': True, 'is_replace': True}],
                 'airport': {'rule': AIRPORT_NAME, 'is_repeated': False, 'is_replace': True},
                 'loyalty': {'rule': FF_REGEX, 'is_repeated': True, 'is_replace': True},
                 'flight_number': {'rule': FLIGHT_NUMBER_REGEX, 'is_repeated': False, 'is_replace': True},
                 'flight_date': {'rule': DATE_PATTERN, 'is_repeated': False, 'is_replace': True},
                 'seat_class': {'rule': FLIGHT_CLASS_REGEX, 'is_repeated': False, 'is_replace': True},
                 'time': {'rule': TIME_PATTERN, 'is_repeated': True, 'is_replace': True},
                 'rewards_program': {'rule': REWARD_PROGRAM, 'is_repeated': False, 'is_replace': False}
                 }

SEGMENT_IDENTIFY_RULES = [[{'rule': TIME_PATTERN, 'segment_rule': AMERICAN_RULE, 'is_validate': True}]]

# It is important to maintain rules order for the airlines.
# It is an extra step to make sure rules must not conflict with any other information of the flight.
# When any of the info matches to rule than we replace matched text with an empty string
# to be on safe side for the next search.
AMERICAN_RULES_ORDER = ['name', 'operating_airline', 'flight_date', 'time', 'flight_number', 'airport', 'seat_class',
                        'rewards_program', 'loyalty']
