"""
This file contains regular expression  that are using in British airlines filter class
"""
import re


def is_more_expansion_needed_segment(flight_info, is_single):
    airport = flight_info.get('airport')
    time = flight_info.get('time')
    flight_number = flight_info.get('flight_number')
    return not (airport and len(airport) > 1 and time and flight_number)


def is_expansion_needed(flight_info, is_single):
    return not ('name' in flight_info)

# To get flight class from string with regex eg : economy or traveller plus or word traveller or domestics
FLIGHT_CLASS_REGEX = re.compile(r"""
    (?xui) # verbose regex case-insensitive
    (?:\|)?\s*(economy|world\s*traveller\s*(?:plus)?|domestic)
    """)

# To match string that look like passenger name
PASSENGER_NAME_REGEX = re.compile(r"""
    (?xi)  # verbose regex, case-insensitive
    ^(?:mr|ms|miss|mrs)\s*([a-z]+\s*[a-z]+)
    """)

# Find passengers name table with regex text eg : Passenger or Passenger(s)
PASSENGER_TARGET_REGEX = re.compile(r"""
   (?xi)   # verbose regex, case-insensitive
    ^
    (passenger) # match wrd passenger
    (\(s\))?$    # optional word (s)
    """)

DATE_FORMAT = ['%d %b %Y']

TIME_PATTERN = re.compile(r"""(?x)(\d{1,2}\s*\:\s*\d{2})""")

AIRPORT_NAME = re.compile(r"""(?x) # Example: 19:25 LHR (Heathrow (London)),  13:00 Edinburgh
                                (?:
                                \d{1,2}\s*:\s*\d{2}\s*
                                )?
                                (
                                    (?:[A-z ,\-\.\\]+)\s+\((?:[A-z ,\-\.\\]+)\)
                                )
                                |
                                (?:
                                    \d{1,2}\s*:\s*\d{2}\s*
                                )
                                (
                                    [A-z ,\-\.\\]{5,}
                                )
                            """)

FLIGHT_NUMBER = re.compile(r"""(?x) # Example: BA1442
                                    ([A-Z]{2}\d+)
                                """)

OPERATING_AIR_LINE = re.compile(r"""(?x)  # Example: British Airways |
                                        (?:[A-Z]{2}\d+)\s*([A-z ]+)
                                        \|
                                """)

DATE_PATTERN = re.compile(r"""(?x)(\d{1,2}\ [A-z]{3}\ \d{4})""")

BRITISH_RULE = {
    'airport': {'rule': AIRPORT_NAME, 'is_repeated': True, 'is_replace': True},
    'flight_number': {'rule': FLIGHT_NUMBER, 'is_repeated': False, 'is_replace': True},
    'operating_airline': {'rule': OPERATING_AIR_LINE, 'is_repeated': False, 'is_replace': True},
    'flight_date': {'rule': DATE_PATTERN, 'is_repeated': False, 'is_replace': True},
    'time': {'rule': TIME_PATTERN, 'is_repeated': True, 'is_replace': True},
    'seat_class': {'rule': FLIGHT_CLASS_REGEX, 'is_repeated': False, 'is_replace': True},
    'is_expansion_needed': is_more_expansion_needed_segment
}

SEGMENT_IDENTIFY_RULES = [[{'rule': TIME_PATTERN, 'segment_rule': BRITISH_RULE, 'is_validate': True},
                           {'rule': TIME_PATTERN, 'segment_rule': BRITISH_RULE, 'is_validate': False}
                          ]]

# It is important to maintain rules order for the airlines.
# It is an extra step to make sure rules must not conflict with any other information of the flight.
# When any of the info matches to rule than we replace matched text with an empty string
# to be on safe side for the next search.
BRITISH_RULE_ORDER = ['airport', 'flight_date', 'time',
                      'seat_class', 'name',
                      'operating_airline', 'flight_number']
