"""
This file contains regular expression  that are using in Frontier airlines filter class
"""
import re
from lola.itinerary_parsing.parsers.email_parsers.rules.airlines import TIME_PATTERN, is_more_expansion_needed_segment


# To get all flight date through regex eg:Depart: Wed, Jan 27, 2016
DATE_REGEX = re.compile(r"""
   (?x)   # verbose regex
   (?:     # non capture
    Depart:\s* #match word 'Depart' follow by character : and zero or more white space
   )
   (.*)    # matches any character (except newline)
    """)

# To get time and flight destination with regex in soup text eg : ATL, ATLANTA, GA
AIRPORT_REGEX = re.compile(r"""
   (?x)
   ([A-Z]{3},\ [A-Z\ \-]+,\ [A-Z]{2})
    """)

# to get flight Seat with regex eg : Leanne Blade
NAME_REGEX = re.compile(r"""
   (?x)   # verbose regex
   (
    [A-z]+\ +[A-z]+\ *[A-z]*
   )
   \s+
   (?:
    \d{2}  # match digit and repeat 2 times
    [A-Z] # match character from range A to Z and repeat 1 time
   )
    """)

# To get Flight number with regex eg: F91472
FLIGHT_NUMBER_REGEX = re.compile(r"""
   (?x)   # verbose regex
   (
    (?:F9|FFT)     # match character 'F'
    \d{3,4} # match digit and repeat 5 times
   )
    """)

DATE_FORMAT = ['%a, %b %d, %Y']

FRONTIER_RULES = {
    'airport': {'rule': AIRPORT_REGEX, 'is_repeated': True, 'is_replace': True},
    'flight_number': {'rule': FLIGHT_NUMBER_REGEX, 'is_repeated': False, 'is_replace': True},
    'name': {'rule': NAME_REGEX, 'is_repeated': True, 'is_replace': True},
    'flight_date': {'rule': DATE_REGEX, 'is_repeated': False, 'is_replace': True},
    'time': {'rule': TIME_PATTERN, 'is_repeated': True, 'is_replace': True},
    'is_expansion_needed': is_more_expansion_needed_segment
}

# Purpose of adding list as an item of a list is to make a group, and from a group only one item can be matched
# per email not more than one at the same time.
SEGMENT_IDENTIFY_RULES = [[{'rule': TIME_PATTERN, 'segment_rule': FRONTIER_RULES, 'is_validate': True}
                           ]]

# It is important to maintain rules order for the airlines.
# It is an extra step to make sure rules must not conflict with any other information of the flight.
# When any of the info matches to rule than we replace matched text with an empty string
# to be on safe side for the next search.
FRONTIER_RULE_ORDER = ['name', 'airport', 'flight_date', 'time', 'flight_number']
