"""
This file contains regular expression  that are using in jetblue airlines filter class
"""
import re
from lola.itinerary_parsing.parsers.email_parsers.rules.airlines import TIME_PATTERN

#  To check if link contain specific word like 'virtuallythere'
FLIGHT_CHARGES_LINK_REGEX = re.compile(r"""
   (?x)   # verbose regex
   (
    virtuallythere  # match word 'virtuallythere'
   )
    """)

# To match flight seat number with regex eg : 11A
FLIGHT_SEAT_REGEX = re.compile(r"""
   (?x)   # verbose regex
   (\d{1,2}[A-Z])
    """)

# To match flight confirmation number|code eg : Your confirmation code is VTOUZK
RECORD_CONFIRMATIONS_REGEX = re.compile(r"""
    (?xi)   # verbose regex, case-insensitive
    (?:confirmation)\s*(?:\#|number|code)\s*(?:is|:)\s*([A-Z]*)
    """)

def is_more_expansion_needed_segment(flight_info, is_single):
    airport_name = flight_info.get('airport')
    name = flight_info.get('name')
    flight_date = flight_info.get('flight_date')
    return not (airport_name and len(airport_name) > 1) or not name or not flight_date

FORWARD_TRAVERSAL_LIMIT = 4

DATE_FORMAT = ['%a, %b %d', '%a,%b %d', '%B %d']

AIRPORT_NAME = re.compile(r"""(?x) # Example : SAN FRANCISCO, CA (SFO)
                                    (
                                        [A-z]+,?\/?\ *[A-z]*,?\/?\ *[A-z]*,?\/?\ *[A-Z]{2}\s+
                                        (?:
                                            \(\s*[A-Z]+\s*\)
                                        )?
                                    )
                                    (?:\s*to\s*)
                                    (
                                        [A-z]+,?\/?\ *[A-z]*,?\/?\ *[A-z]*,?\/?\ *[A-Z]{2}\s+
                                        (?:
                                            \(\s*[A-Z]+\s*\)
                                        )?
                                    )
                                """)

AIRPORT_CODE = re.compile(r"""(?x) # Example : (SFO)
                              [A-Z]+,?\ [A-Z]*,?\ *[A-Z]*,?\ *[A-Z]{2}\s+
                              \(\s*([A-Z]+)\s*\)
                              """)

DATE_PATTERN = re.compile(r"""(?x) # Example : Wed, Aug 12 | Wed, Apr 14 - Thu, Apr 15 | August 20 - August 21
                                (?:
                                    (
                                        [A-z]{3},\s+[A-z]{3}\ \d{1,2}|[A-z]+\ \d{1,2}\-[A-z]+\ \d{1,2}

                                    )
                                    (?:\s*\-\s*)?
                                    (
                                        [A-z]{3},\s+[A-z]{3}\ \d{1,2}|[A-z]+\ \d{1,2}\-[A-z]+\ \d{1,2}

                                    )?
                                )
                                |
                                (?: # Excluded, so that it won't appear as a duplicate in searched result set
                                    (
                                        [A-z]+\ *\d{2}
                                    )
                                    \s*\-\s*
                                    (
                                        [A-z]+\ *\d{2}
                                    )
                                )
                                """)

FLIGHT_NUMBER = re.compile(r"""(?x)
                                (\b\d{2,4}\b)
                           """)

PASSENGER_NAME_PATTERN = re.compile(r"""
                                        (?x)
                                        ([A-z]+\ +[A-z]+\ *[A-z]*\ *[A-z]*)
                                        \s*
                                        (?:B\d|N/A)
                                    """)

PASSENGER_NAME_PATTERN2 = re.compile(r"""
                                        (?x)
                                        (?:\d{3,4}\s*)
                                        ([A-z]+\ +[A-z]+\ *[A-z]*\ *[A-z]*)
                                        \s*|([A-z]+\ +[A-z]+\ *[A-z]*\ *[A-z]*)(?:\s*\d{1,2}[A-Z])
                                    """)

# Following are keywords under category.
# keywords are used to categorize purchase amount into different categories
CHARGE_PATTERN = {'fees': ['(?<!\-\s)fee'],
                  'tax': ['tax'],
                  'base': ['fare'],
                  'total': ['total'],
                  'exclude': []}

REWARD_PROGRAM = re.compile(r"""^(?x)(Frequent\s*Flyer)$""")

REWARD_ID = re.compile(r"""(?x)(B\d\s+\d+)""")

JETBLUE_RULE = {
    'airport': {'rule': AIRPORT_NAME, 'is_repeated': False, 'is_replace': False},
    'airport_code': {'rule': AIRPORT_CODE, 'is_repeated': False, 'is_replace': True},
    'flight_number': {'rule': FLIGHT_NUMBER, 'is_repeated': False, 'is_replace': True},
    'name': [{'rule': PASSENGER_NAME_PATTERN, 'is_repeated': True, 'is_replace': True, 'match_again': False},
             {'rule': PASSENGER_NAME_PATTERN2, 'is_repeated': True, 'is_replace': True, 'match_again': True}],
    'flight_date': {'rule': DATE_PATTERN, 'is_repeated': True, 'is_replace': True},
    'time': {'rule': TIME_PATTERN, 'is_repeated': True, 'is_replace': True},
    'loyalty': {'rule': REWARD_ID, 'is_repeated': True, 'is_replace': True},
    'flight_seat': {'rule': FLIGHT_SEAT_REGEX, 'is_repeated': True, 'is_replace': True},
    'is_expansion_needed': is_more_expansion_needed_segment
}

SEGMENT_IDENTIFY_RULES = [[{'rule': TIME_PATTERN, 'segment_rule': JETBLUE_RULE, 'is_validate': True}]]

# It is important to maintain rules order for the airlines.
# It is an extra step to make sure rules must not conflict with any other information of the flight.
# When any of the info matches to rule than we replace matched text with an empty string
# to be on safe side for the next search.
JETBLUE_RULE_ORDER = ['time', 'flight_date', 'airport', 'airport_code', 'seat_class', 'name', 'flight_number',
                      'loyalty', 'rewards_program', 'flight_seat']
