"""
This file contains regular expression  that are using in lufthansa airlines filter class
"""
import re
from lola.itinerary_parsing.parsers.email_parsers.rules.airlines import TIME_PATTERN, is_more_expansion_needed_segment

# To get flight passenger names table
NAME_REGEX = re.compile(r"""
   (?x)   # verbose regex
    Ticket\sno  # match word 'Ticket no'
    """)

FORWARD_TRAVERSAL_LIMIT = 5

DATE_FORMAT = ['%a %d %B %Y']

AIRPORT_NAME = re.compile(r"""(?x) # Example : CHICAGO IL O HARE INTERNATIONAL (ORD)
                                (
                                    [A-z ,]+\(\s*[A-z]+\s*\)
                                )
                                """)

AIRPORT_CODE = re.compile(r"""(?x)(?: # Example : CHICAGO IL O HARE INTERNATIONAL (ORD)
                                   [A-z ,]+
                                     \(\s*
                                     ([A-z]+)
                                     \s*\)
                                   )
                                   """)

FLIGHT_NUMBER = re.compile(r"""(?x) # Example: OSO66 operated by
                                (
                                    [A-Z]+\d+
                                )
                                \s*
                                (?:
                                    operated\s*by
                                )
                                """)

PASSENGER_NAME_PATTERN = re.compile(r"""
                                    (?x) # Example: BELL / LEANDRA MS
                                    (
                                        [A-Z]+\s*(?:\/)?\s*[A-Z]+\s*[A-Z]*\s*
                                     )
                                     (?:
                                         MS|MR
                                     )

                                """)

DATE_PATTERN = re.compile(r"""
                            (?x)
                            (
                                [A-z]{3}\.\ \d{1,2}\ [A-z]+\ \d{4}
                            )
                            """)

CABIN_TYPE_REGEX = re.compile(r"""
                            (?x) # Example: Class: ECONOMY (T)
                            (?:
                                Class:\s*(.*\([A-Z]\))
                            )
                            """)

OPERATING_AIR_LINE = re.compile(r"""(?x) # Example : operated by: AUSTRIAN AIRLINES
                                (?:
                                    operated\s*by\s*:\s*
                                )
                                ([A-Z ]+)
                                """)
LUFTHANSA_RULE = {
    'airport': {'rule': AIRPORT_NAME, 'is_repeated': False, 'is_replace': False},
    'airport_code': {'rule': AIRPORT_CODE, 'is_repeated': False, 'is_replace': True},
    'flight_number': {'rule': FLIGHT_NUMBER, 'is_repeated': False, 'is_replace': True},
    'operating_airline': {'rule': OPERATING_AIR_LINE, 'is_repeated': False, 'is_replace': True},
    'flight_date': {'rule': DATE_PATTERN, 'is_repeated': False, 'is_replace': True},
    'time': {'rule': TIME_PATTERN, 'is_repeated': True, 'is_replace': True},
    'seat_class': {'rule': CABIN_TYPE_REGEX, 'is_repeated': False, 'is_replace': True},
    'is_expansion_needed': is_more_expansion_needed_segment
}

PASSENGER_SEGMENT_RULE = {
    'name': {'rule': PASSENGER_NAME_PATTERN, 'is_repeated': True, 'is_replace': True, 'match_again': False}
}

PASSENGER_IDENTIFY_RULE = re.compile(r"""(?x)(Passenger\s*Information)""")

SEGMENT_IDENTIFY_RULES = [[{'rule': TIME_PATTERN, 'segment_rule': LUFTHANSA_RULE, 'is_validate': True},
                           ], [{'rule': NAME_REGEX, 'segment_rule': PASSENGER_SEGMENT_RULE,
                               'is_validate': False}]]

# It is important to maintain rules order for the airlines.
# It is an extra step to make sure rules must not conflict with any other information of the flight.
# When any of the info matches to rule than we replace matched text with an empty string
# to be on safe side for the next search.
LUFTHANSA_RULE_ORDER = ['time', 'flight_date', 'airport',
                        'airport_code', 'seat_class', 'name', 'flight_number', 'loyalty', 'rewards_program',
                        'operating_airline', 'flight_seat']
