"""
This file contains regular expression  that are using in southwest airlines filter class
"""
import re
from lola.itinerary_parsing.parsers.email_parsers.rules.airlines import TIME_PATTERN, is_more_expansion_needed_segment
FORWARD_TRAVERSAL_LIMIT = 2


def is_more_expansion_needed(flight_info, is_single):
    return not 'name' in flight_info

DATE_FORMAT = ['%a %b %d']

AIRPORT_NAME = re.compile(r"""(?x) # Example: 'Depart BOSTON LOGAN, MAA (BOS)'
                                (?:Depart|Arrive in|in) # Start with any of the mentioned keyword
                                \s*     # Followed by a space repeats zero or more
                                (
                                    [A-z]+  # Followed by letters A-z repeats one or more
                                    \/?\ *  # Followed by slash optional and space repeats zero or more time
                                    [A-z]+\/?  # Followed by letters A-z repeats one or more and options character slash
                                    ,?\ *  # optional character comma
                                    [A-z]*  # Letters A-z repeats zero or more
                                    \s*
                                    \(.*\)
                                )
                                """)

AIRPORT_CODE = re.compile(r"""(?x)  # Example: Arrive in ATLANTA, GAA (ATL)
                              (?:Depart|Arrive in|in)
                              \s*
                              (?:
                                  [A-z]+\/?\ *
                                  [A-z]+\/?,?\ *
                                  [A-z]*\s*
                              )
                              \((.*)\)
                              """)

FLIGHT_NUMBER = re.compile(r"""(?x) # Example:  '573  Depart'
                            (
                                \d{3,4})\s+(?:Depart|Change
                            )
                            """)

DATE_PATTERN = re.compile(r""" (?x) # Example 'Sun Nov 15'
                           (
                            [A-z]{3}\ [A-z]{3}\ \d{1,2}
                           )
                           """)

PASSENGER_IDENTIFY_RULE = re.compile(r"""
   (?xi)   # verbose regex, case-insensitive
    (passenger\(s\))  # match word 'Passenger(s)'
    """)

PASSENGER_NAME_PATTERN = re.compile(r"""(?x) # Example: 'MORGAN/DOUGLAS SCOTT'
                                        (
                                            [A-Z]+
                                            (?:\/|\ )
                                            [A-Z]+\s+
                                            (?:[A-Z]{3,})* # Non capture group. Capital letters should have length greater
                                                           # than or equal to 3 but could be optional
                                        )
                                        """)
REWARD_ID = re.compile(r'\s+\d+\s+')
REWARD_PROGRAM = re.compile(r'Rapid Rewards \#')

SOUTHWEST_RULE = {
    'airport': {'rule': AIRPORT_NAME, 'is_repeated': False, 'is_replace': False},
    'airport_code': {'rule': AIRPORT_CODE, 'is_repeated': False, 'is_replace': True},
    'flight_number': {'rule': FLIGHT_NUMBER, 'is_repeated': False, 'is_replace': True},
    'flight_date': {'rule': DATE_PATTERN, 'is_repeated': True, 'is_replace': True},
    'time': {'rule': TIME_PATTERN, 'is_repeated': True, 'is_replace': True},
    'is_expansion_needed': is_more_expansion_needed_segment
}


PASSENGER_RULES = {
    'name': [{'rule': PASSENGER_NAME_PATTERN, 'is_repeated': True, 'is_replace': True, 'match_again': True}],
    'rewards_program': {'rule': REWARD_PROGRAM, 'is_repeated': False, 'is_replace': True},
    'is_expansion_needed': is_more_expansion_needed}

SEGMENT_IDENTIFY_RULES = [[{'rule': TIME_PATTERN, 'segment_rule': SOUTHWEST_RULE, 'is_validate': True}],
                          [{'rule': PASSENGER_IDENTIFY_RULE, 'segment_rule': PASSENGER_RULES, 'is_shift': False}]]

# It is important to maintain rules order for the airlines.
# It is an extra step to make sure rules must not conflict with any other information of the flight.
# When any of the info matches to rule than we replace matched text with an empty string
# to be on safe side for the next search.
SOUTHWEST_RULE_ORDER = ['time', 'flight_number', 'flight_date', 'airport',
                      'airport_code', 'seat_class', 'name', 'loyalty', 'transaction_id', 'rewards_program']
