"""
This file contains regular expression  that are using in spirit airlines filter class
"""
import re
from lola.itinerary_parsing.parsers.email_parsers.rules.airlines import TIME_PATTERN, is_more_expansion_needed_segment

# To get flight date list eg: TUESDAY, FEBRUARY 19, 2016
FLIGHT_DATE_REGEX = re.compile(r"""
   (?x)   # verbose regex
    (
     [A-z]* # match zero or more character from range A to Z
     \,     # match character ,
     \s+    # match one or more white space
     [A-z]* # match zero or more character from range A to Z
     \s     # match white space
     \d{1,2}    # match  digits [0-9] and repeat 1 to 2 times
     \,     # match character ,
     \s+    # match one or more white space
     \d{4}  # match digit and repeat 4 times
    )
    """)

# To parse flight destination name and time eg : Boston, MA 4:00 PM
FLIGHT_DESTINATION_REGEX = re.compile(r"""
   (?x)   # verbose regex
    \s*     # match zero or more white space
    (
     [A-z\s]+ # match zero or more character from range A to Z and a t to z or white space
     ,     # match character ,
     \s+    # match one or more white space
     [A-Z]* # match zero or more character from range A to Z
     \s+    # match one or more white space
    )
    (?:\d{1,2}:\d{1,2}\s+[AM|PM]+) # match  digits [0-9] and repeat 1 to 2 times eg: 4:00 PM
    """)

# To parse flight number from text eg : 465 B
FLIGHT_NUMBER_REGEX = re.compile(r"""
   (?xus)   # verbose regex, unicode, dotall regex
    ^       # assert position at start of the string
    \s*     # match zero or more white space
    (\d{3,}) # match digit and repeat 3 times
    """)

SEGMENT_IDENTIFY_RULE = re.compile(r"""
                            ^(?xi)  # verbose regex, case-insensitive
                            (?:
                                (
                                    \d{1,2}:    # match digits and repeat 1 to 2 times follow by character :
                                    \d{2}\s*[ap].?[m] # match digits and repeat two times follow by zero or more space and
                                                    # character am or pm
                                )
                                |
                                (
                                    \d{1,2}:
                                    \d{2}\s*
                                )
                                [nh]
                            )\b
                          """)

DATE_FORMAT = ['%A, %B %d, %Y']
FORWARD_TRAVERSAL_LIMIT = 6

SPIRIT_RULES = {
    'airport': {'rule': FLIGHT_DESTINATION_REGEX, 'is_repeated': True, 'is_replace': True},
    'flight_number': {'rule': FLIGHT_NUMBER_REGEX, 'is_repeated': False, 'is_replace': True},
    'flight_date': {'rule': FLIGHT_DATE_REGEX, 'is_repeated': True, 'is_replace': True},
    'time': {'rule': TIME_PATTERN, 'is_repeated': True, 'is_replace': True},
    'is_expansion_needed': is_more_expansion_needed_segment
}

PASSENGER_NAME_REGEX = re.compile(r"""
                                        (?x) # Example: LEANNE BLADE
                                            (
                                                [A-Z\\\-]{3,}\ +[A-Z\\\-]{3,}
                                            )
                                            \s+
                                            (?:
                                                None\b
                                            )
                                        """)

REWARD_ID = re.compile(r"""(?x)\b
                                (?:None\s+)
                                (\d+)
                                \b
                                """)

REWARD_PROGRAM = re.compile(r"""(?x)
                                    (FREE\ +SPIRIT\ +)
                                    (?:\#)
                                """)

PASSENGERS_RULE = {
    'name': {'rule': PASSENGER_NAME_REGEX, 'is_repeated': True, 'is_replace': True},
    'loyalty': {'rule': REWARD_ID, 'is_repeated': True, 'is_replace': True},
    'rewards_program': {'rule': REWARD_PROGRAM, 'is_repeated': False, 'is_replace': True}
}

PASSENGER_SEGMENT_IDENTIFIER = re.compile(r"""^(?xi)\s*Customer\sInformation\s*$""")

# Purpose of adding list as an item of a list is to make a group, and from a group only one item can be matched
# per email not more than one at the same time.
SEGMENT_IDENTIFY_RULES = [[{'rule': SEGMENT_IDENTIFY_RULE, 'segment_rule': SPIRIT_RULES, 'is_validate': False}
                           ], [{'rule': PASSENGER_SEGMENT_IDENTIFIER, 'segment_rule': PASSENGERS_RULE,
                                'is_validate': False}, ]]

# It is important to maintain rules order for the airlines.
# It is an extra step to make sure rules must not conflict with any other information of the flight.
# When any of the info matches to rule than we replace matched text with an empty string
# to be on safe side for the next search.
SPIRIT_RULE_ORDER = ['name', 'rewards_program', 'loyalty',  'airport', 'flight_date', 'time', 'flight_number']
