"""
This file contains regular expression  that are using in united airlines filter class
"""
import re
from lola.itinerary_parsing.parsers.email_parsers.rules.airlines import TIME_PATTERN


def is_more_expansion_needed(flight_info, is_single):
    return not 'name' in flight_info


def is_more_expansion_needed_segment(flight_info, is_single):
    return is_single and not 'flight_date' in flight_info


DATE_FORMAT = ['%d%b%y', '%a, %b %d, %Y']

# To match passenger name pattern. There are three alternative eg : 'Traveler MORGAN/DOUGLAS' or
# 'Christian Busch SFO to DEN' or 'Ms. Rachel Neasham Seats: BOS - SFO: 5B'
PASSENGER_NAME_PATTERN = re.compile(r"""
    (?x)        # verbose regex
    (
        [A-Z]+  # match one or more than one character from range [A-Z]
        \s*     # match zero roe more white space character
        \/      # matches the character / literally
        [A-Z]+  # match one or more than one character from range [A-Z]
    )
    |           # Alternative
    (
        [A-z]{3,}   # match three or more than three character from range [A-z]
        \ +         # match more than one space
        [A-z]{3,}   # match three or more than three character from range [A-z]
        \ *         # match zero or more white space character
        [A-z]*      # match zero or more than one character from range [A-z]
        \ *         # match zero or more white space character
        [A-z]*      # match zero or more than one character from range [A-z]
    )\s+(?:Seats|[A-Z]{3}\s+(?:to|\-)\s+[A-Z]{3})   # Non-capturing group
    |           # Alternative
    (?:Traveler\ Information\s*)(?:,\s*)?   # Non-capturing group
    (
        [A-z]{3,}   # match three or more than three character from range [A-z]
        \ +         # match more than one space
        [A-z]{3,}   # match three or more than three character from range [A-z]
        \ *         # match zero or more white space character
        [A-z]*      # match zero or more than one character from range [A-z]
        \ *         # match zero or more white space character
        [A-z]*      # match zero or more than one character from range [A-z]
    )(?:\s*,)?  # Non-capturing group (match zero or more white space character follow by character , optional)
    """)

# To match airport names like 'Denver, CO, US (DEN)'
AIRPORT_NAME = re.compile(r"""
    (?x)    # verbose regex
    (
        [A-z]+  # match one or more than one character from range [A-z]
        ,?\s    # match character , as optional follow by white space character
        [A-z]+  # match one or more than one character from range [A-z]
        ,?\s?   # match character , as optional follow by white space character optional
        \s[A-z]*    # a white space character and zero or more character from range [A-z]
        \s*     # match zero or more white space character
    )
    \((?:.*)\)  # Non-capturing group (match any thing within brackets except new line)
    """)

# To match airport code from string eg: 'BOSTON, MA  (BOS)'
AIRPORT_CODE = re.compile(r"""
    (?x)        # verbose regex
    (?:         # Non-capturing group
        [A-z]+  # match one or more than one character from range [A-z]
        ,?\s    # match character , as optional follow by white space character
        [A-z]+\s* # match one or more character from range [A-z] follow by white space character
    )
    \((.*)\)    # match any thing within brackets except new line
    """)

# To match flight class name from sting eg : ' T ' or 'United First'
FLIGHT_CLASS_REGEX = re.compile(r"""
    (?x)        # verbose regex
    (\s[A-Z]\s) # match a white space character then character from range [A-Z] follow by white space character
    |           # Alternative
    (?:United\s*)# Non-capturing group (match character United follow by white space character)
    ([A-z]+)    # match one or more character from range [A-z]
    """)

# To match flight number eg : UA 2014
FLIGHT_NUMBER = re.compile(r"""
    (?x)        # verbose regex
    (
        UA\s?   # match characters UA follow by optional white space character
        [A-Z]*  # match zero or more than one character from range [A-Z]
        \s?     # match optional white space character
        \d{1,4} # match digit and repeat from one to four
    )
    """)

# To match date pattern from string eg:'Thu, 11FEB16' or 'Thu, Dec 17, 2015'
DATE_PATTERN = re.compile(r"""
    (?x)        # verbose regex
    (\d{1,2}[A-Z]{3}\d{2})  # match one to two digits follow by three character from range [A-Z] and then match two digits
    |           # Alternative
    ([A-z]{3}.?,\ [A-z]{3}.?\ \d{1,2},\ \d{4}) # three characters from [A-z] follow by optional character . and ,
                                            # repeat that match  then match one to two digits follow by character ,
                                            # and then match four digits
    """)

PASSENGER_IDENTIFY_RULE = re.compile(r"""
    (?x)        # verbose regex
    ^           # assert position at start of the string
    \s*         # match any white space character
    Travelers?  # match optional word 'Travelers'
    \s*         # match any white space character
    (Details|Information)?  # match optional words Details or Information case sensitive
    $           # assert position at end of the string
    """)

# To match operating airline name from string eg : Flight operated by SKYWEST AIRLINES
OPERATING_AIR_LINE = re.compile(r"""
    (?x)        # verbose regex
    (?:Flight\ operated\ by\ ) # Non-capturing group (match words 'Flight operated by ')
    ([A-Z ]+)   # match one or more character from [A-Z] and space
    """)

# To match plane type from string eg : '737-900' or 'CRJ-700'
PLANE_TYPE_PATTERN = re.compile(r"""
    (?x)        # verbose regex
    (
        [A-Z]{0,3}  # match zero to three character from [A-Z]
        \d*?        # match zero or more digit optional
        \-          # match character -
        \d+         # match one or more digit
    )
    """)

SEGMENT_IDENTIFY_PATTERN_AIRPORT = re.compile(r"""
    (?x)        # verbose regex
    ^           # assert position at start of the string
    \((.*)\)    # match any thing within brackets except new line
    $           # assert position at end of the string
    """)

# To match flight frequent flyer number like 'UA-****391' or 'UA-XXXX391' or 'TK-****391'
FF_REGEX = re.compile(r"""
    (?x)        # verbose regex
    \s*         # match zero or more white space character
    (
        (?:UA|TK)(?:\-)(?:[\*A-z0-9]+)
    )
    \s*         # match zero or more white space character
    """)

# to match date
DATE_PATTERN2 = re.compile(r"""
    (?x)        # verbose regex
    (
        ^       # assert position at start of the string
        [A-z]{3}.?,\ [A-z]{3}.?\ \d{1,2},\ \d{4}    # match three character from [A-z] follow by character . optional
                                                    # then match character , and match again that pattern [A-z]{3}.?
                                                    # then match one to two digit follow by character , and space
                                                    # then match four digits
        $       # assert position at end of the string
    )
        """)
# To match flight reward program name like 'Frequent Flyer'
REWARD_PROGRAM = re.compile(r"""
    (?xi)      # verbose regex, case-insensitive
    (\s*frequent\ flyer\s*)   # match word Frequent Flyer
    """)

UNITED_RULE = {
    'airport': {'rule': AIRPORT_NAME, 'is_repeated': False, 'is_replace': False},
    'airport_code': {'rule': AIRPORT_CODE, 'is_repeated': False, 'is_replace': True},
    'flight_number': {'rule': FLIGHT_NUMBER, 'is_repeated': False, 'is_replace': True},
    'operating_airline': {'rule': OPERATING_AIR_LINE, 'is_repeated': False, 'is_replace': True},
    'flight_date': {'rule': DATE_PATTERN, 'is_repeated': True, 'is_replace': True},
    'seat_class': {'rule': FLIGHT_CLASS_REGEX, 'is_repeated': False, 'is_replace': True},
    'time': {'rule': TIME_PATTERN, 'is_repeated': True, 'is_replace': True},
    'plane_type': {'rule': PLANE_TYPE_PATTERN, 'is_repeated': False, 'is_replace': True},
    'is_expansion_needed': is_more_expansion_needed_segment
}

PASSENGER_RULES = {
    'name': [{'rule': PASSENGER_NAME_PATTERN, 'is_repeated': True, 'is_replace': True, 'match_again': True}],
    'rewards_program': {'rule': REWARD_PROGRAM, 'is_repeated': False, 'is_replace': True},
    'loyalty': {'rule': FF_REGEX, 'is_repeated': True, 'is_replace': True},
    'is_expansion_needed': is_more_expansion_needed}

SEGMENT_IDENTIFY_RULES = [[{'rule': TIME_PATTERN, 'segment_rule': UNITED_RULE, 'is_validate': True},
                           {'rule': SEGMENT_IDENTIFY_PATTERN_AIRPORT, 'segment_rule': UNITED_RULE, 'is_validate': True,
                            'is_shift': False}],
                          [{'rule': PASSENGER_IDENTIFY_RULE, 'segment_rule': PASSENGER_RULES, 'is_shift': False}]]

# It is important to maintain rules order for the airlines.
# It is an extra step to make sure rules must not conflict with any other information of the flight.
# When any of the info matches to rule than we replace matched text with an empty string
# to be on safe side for the next search.
UNITED_RULES_ORDER = ['time', 'plane_type', 'operating_airline', 'flight_date', 'flight_number', 'airport',
                      'airport_code', 'seat_class', 'rewards_program', 'name', 'loyalty']
