"""
This file contains regular expression that are using in Virgin america airlines filter class
"""
import re
from lola.itinerary_parsing.parsers.email_parsers.rules.airlines import TIME_PATTERN, is_more_expansion_needed_segment


def is_more_expansion_needed(flight_info, is_single):
    return not flight_info


FORWARD_TRAVERSAL_LIMIT = 5
DATE_FORMAT = ['%d%b%Y', '%d-%b-%Y']

SEGMENT_IDENTIFY_RULE = re.compile(r"""
    (?x)        # verbose regex
    (
        [A-z]+,?    # match one or more character from [A-z] follow by optional character ,
        \s          # match single white space character
        [A-Z]+\s    # match one or more character from [A-Z] follow by single white space character
        \([A-Z]{2,3}\)  # match two to three character from [A-Z] within brackets ()
    )
    """)

PASSENGER_IDENTIFY_RULE = re.compile(r"""
    (?x)        # verbose regex
    (Travelers?)    # match word 'Traveler' with optional character s
    (
        \s\d:       # match single white space character follow by a digit with special character :
        |           # Alternative
        \(s\)       # match character 's' within brackets ()
        $           # assert position at end of the string
    )
    """)

# Rule to match date like '21-Feb-2014'
DATE_RULE1 = re.compile(r"""
    (?x)        # verbose regex
    \d{1,2}\-[A-Z][a-z]{2}\-\d{4} # match one to two digits follow by character -
                                  # then match single character from [A-Z] follow by two character from [a-z]
                                  # then match character - follow by four digits
    """)
# Rule to match date like '01Mar2016'
DATE_RULE2 = re.compile(r"""
    (?x)        # verbose regex
    \d{1,2}[A-Z][a-z]{2}\d{4}   # match one to two digits then match single character from [A-Z] follow by two
                                # character from [a-z] then match four digits
    """)

# To match flight numbers from string eg : VX 351
FLIGHT_NUMBER_REGEX = re.compile(r"""
    (?x)        # verbose regex
    (
        [A-Z]{2}\s* # match two character from [A-Z] follow by zero or more white space
        \d{2,3}     # match two to three digits
    )
    """)
# To match passenger names like 'Traveler 1: PAUL ENGLISH' or 'Traveler(s) PAUL ENGLISH'
PASSENGER_NAME_PATTERN1 = re.compile(r"""
    (?x)        # verbose regex
    (?:
        Travelers?\s*\(?s?\)?\s*\d?\s*:? # match word Traveler with optional character like 's' and  '(' ')' and
                                         # white space and digit and :
    )
    \s*                                  # match zero or more white space character
    ([A-Z\ ]+)                           # match one or more space and character from [A-Z]
    (?![a-z]) # Negative Lookahead - Assert that it is impossible to match the regex [a-z]
    """)

# To match passenger names like 'SAN PAUL'
PASSENGER_NAME_PATTERN2 = re.compile("""
    (?x)        # verbose regex
    ([A-Z]+\s[A-Z]+)    # match one or more character from [A-Z] follow by white space then match one or more character
                        # from [A-Z]
    (?![a-z])   # Negative Lookahead - Assert that it is impossible to match the regex [a-z]
    """)

# To match loyalty program id eg : Frequent Flyer: 91640004433
LOYALTY_PATTERN = re.compile(r"""
    (?x)        # verbose regex
    (?:
        Frequent\ Flyer:\s+ # match word 'Frequent Flyer' follow by character : and one white space character
        |   # Alternative
        \s+ # match one or more white space character
    )
    (\d{4,})    # match four plus digit
    (?![A-z])   # Negative Lookahead - Assert that it is impossible to match the regex [A-z]
    """)

# To match flight reward program name like 'Frequent Flyer'
REWARD_PROGRAM = re.compile(r"""
    (?xi)      # verbose regex, case-insensitive
    (\s*frequent\ flyer\s*)   # match word Frequent Flyer
    """)

VIRGIN_AMERICA_RULE = {'airport': {'rule': SEGMENT_IDENTIFY_RULE, 'is_repeated': False, 'is_replace': True},
                       'flight_number': {'rule': FLIGHT_NUMBER_REGEX, 'is_repeated': False, 'is_replace': True},
                       'flight_date': [{'rule': DATE_RULE1, 'is_repeated': False, 'is_replace': True},
                                       {'rule': DATE_RULE2, 'is_repeated': False, 'is_replace': True}],
                       'time': {'rule': TIME_PATTERN, 'is_repeated': True, 'is_replace': True},
                       'is_expansion_needed': is_more_expansion_needed_segment}

VIRGIN_AMERICA_RULE_PASSENGER = {'name': [{'rule': PASSENGER_NAME_PATTERN1, 'is_repeated': True, 'is_replace': True},
                                          {'rule': PASSENGER_NAME_PATTERN2, 'is_repeated': True, 'is_replace': True}],
                                 'rewards_program': {'rule': REWARD_PROGRAM, 'is_repeated': False, 'is_replace': False},
                                 'loyalty': {'rule': LOYALTY_PATTERN, 'is_repeated': True, 'is_replace': True},
                                 'is_expansion_needed': is_more_expansion_needed}

SEGMENT_IDENTIFY_RULES = [[{'rule': SEGMENT_IDENTIFY_RULE, 'segment_rule': VIRGIN_AMERICA_RULE, 'is_shift': False}],
                          [{'rule': PASSENGER_IDENTIFY_RULE, 'segment_rule': VIRGIN_AMERICA_RULE_PASSENGER,
                            'is_shift': True}]]

VIRGIN_AMERICA_RULE_ORDER = ['airport', 'flight_number', 'flight_date', 'time', 'name', 'rewards_program', 'loyalty']
