import re
import regex

# Rules to extract amount for purchase
PATTERN_PURCHASE_AMOUNT = regex.compile(r"""(?xu)   # Sample matched pattern USD dollars $84.00 USD
    (?:(?<=:)|^)\s*        # Should start with zero or more space
    (\p{Sc})?\s*           # match currency symbol optional followed by a space repeats 0 or more times
    ([A-Z]{3})?\s*         # match character from range [A-Z] and repeat three times
    (([A-z ]*)\s(?i)dollars?)?\s* # match character from range [A-z] or a space  zero or more time followed by a
                                  # space and 'dollars' keyword case insensitive
    (\p{Sc})?\s*           # match currency symbol optional followed by a space repeats 0 or more times
    (([A-z ]*)\s(?i)dollars?)?\s*
    \s*         # Followed a space repeats 0 or more times
    (\d+)       # Followed by digit repeats one or more times
    (,\d{3})*  # Followed by pattern of a comma with 3 digits together repeats 0 or more times
    (?:\.\d{2}) # Followed by pattern of dot and 2 digits together shouldn't be part
                # of matched string

        \s*     # Followed by a space repeats zero or more times
        (
            \(? # Followed by bracket optional, there might be a case when purchase amount
                # is enclosed in brackets therefore making it optional
                ([A-Z]{3}|([A-z ]*)\s(?i)dollars?)  # Capture capital letters of length 3 or letters or a space repeats
                                                    # zero or more time followed by a
                                                    # space and 'dollars' keyword case insensitive
            \)?
        )?  # Currency part sometimes is not under same element could be collected from sibling element,
            # therefore making it as optional
       # Group to capture currency code out of purchase amount
    (\*|\s|\-)*$ # string should ends with a * or a space repeats zero or more times
                              """)

PATTERN_PURCHASE_AMOUNT_ONLY = regex.compile(r"""(?xu)^\s*(\p{Sc}\s*\d+)\s*$ """)

# Pattern to detect currency used for purchase
PATTERN_CURRENCY_AFTER = re.compile(r"""
                              (?xsu) # Sample matched pattern 230.00 USD
                              (?<!\+ )  # Doesn't consume character further if + or spaces matches before any character
                              (?:\d{2,})   # Followed by a digit repeats 2 or more times should not suppose to be present in matched pattern,
                                        # since we are only interested in pulling currency code out of string
                              (?:       # should not suppose to be present in matched results
                                ,\d{3}     # Followed by a pattern of comma together with 3 digits optional,
                              )?    # optional
                              (?:   # should not suppose to be present in matched results
                                \.\d{2} # Followed by pattern of dot together with 2 digits occurs only once
                              )
                              \s{1,5}   # Followed by a space repeats 1 - 3 times
                              (?!per)   # Doesn't consume characters if a string contains 'per' keyword
                                        # ahead of matched pattern
                              (?: # should not suppose to be present at matched pattern
                                \(? # Followed by optional bracket shouldn't suppose to be included in matched pattern
                              )
                              (
                                ([A-Z]{3}|([A-z ]*)\s(?i)dollars?)    # Capture capital letters of length 3 or letters or a space repeats
                                                                      # zero or more time followed by a
                                                                      # space and 'dollars' keyword case insensitive
                              ) # Capturing group since we are interested in pulling currency code out of matched pattern
                              (?:   # Should not suppose to be a part of matched pattern
                                \)? # an optional bracket because purchase amount might enclosed in a bracket
                              )
                              (?![A-z])
                              """)

PATTERN_CURRENCY_BEFORE = re.compile(r"""(?xu)   # Sample matched pattern (USD) 233.00
                                           # This pattern is different from previous pattern in a way that there might be a chance
                                           # currency code comes before purchase amount
                                   (?:  # should not suppose to be a part of match results since we are interested
                                        # in pulling currency code out of purchase amount
                                    \(? # Followed by a bracket optional
                                   )
                                   (?<![A-z]) # Negative look behind, drop matched pattern ahead if any letters A-z comes before it.
                                   (
                                    ([A-Z]{3}|([A-z ]*)\s(?i)dollars?)    # Capture capital letters of length 3 or letters or a space repeats
                                                                          # zero or more time followed by a
                                                                          # space and 'dollars' keyword case insensitive
                                   )
                                   (?:  # Should not suppose to be a part of matched results
                                    \)? # Followed by a bracket optional
                                   )
                                   \s*  # Followed by a space repeats zero or more times
                                   (?:  # Should not suppose to be a part of matched results
                                    \d{2,} # Followed by a digit repeats 2 or more times
                                    )
                                   (?:  # Should not suppose to be a part of matched results
                                   ,\d{3}  # Followed by a pattern of comma together with 3 digits optional,
                                   )?   # optional
                                   (?:  # Should not suppose to be a part of matched results
                                    \.\d{2} # Followed by pattern of dot together with 2 digits occurs only once
                                   )
                                   (?!\%)   # Doesn't consume characters if a string contains '!' tokens
                                            # ahead of matched pattern
                                   """)

# Following are keywords under category.
# keywords are used to categorize purchase amount into different categories
CHARGE_PATTERN = {'fees': ['fee', 'charge'],
                  'tax': ['tax', 'ccf', 'state', 'local', '%'],
                  'base': ['per', 'rate', 'night', 'room', 'reservation'],
                  'total': ['total', 'subtotal'],
                  'exclude': []}

# Rule to extract participant name from email
PATTERN_PARTICIPANTS = re.compile(r"""(?x)  # Sample matched pattern Ms. Participant name
                                      (?:   # Should not suppose to be a part of matched pattern
                                        Ms\.?|  # Followed by Ms and a optional dot
                                        Mrs?\.?| # Or a Mr and and a optional dot
                                        booking with us,? # Or a mentioned pattern followed by a optional commma
                                      )
                                      (
                                        [a-zA-z ]+    # Followed by letter small/capital or a space repeats
                                                        # one or more times
                                      )
                                      (?:   # Should not suppose to be a part of matched pattern
                                        ,|  # Followed by a comma or a space
                                      )? # optional
                                      """)

PATTERN_PARTICIPANTS2 = re.compile(r"""(?xi)  # Sample matcher Pattern: name, see you on
                                        ( # Should be part of result set
                                            ([A-z]+)    # it could be Letter small/capital A-z, repeats one or more times
                                            \s?  # Followed by a space
                                            (
                                                [A-z]+ # Followed by Letter small/capital A-z, repeats one or more times
                                            )? # Optional
                                        )
                                        ,   # Followed by a ','
                                        \s* # Followed by a space repeats zero or more times
                                        (?= # Positive look ahead
                                            see\s*you\s*on  # Match previous mentioned pattern only if this pattern
                                                            # lies ahead of it
                                        )
                                      """)


# Rule to extract no of guests from email
PATTERN_ADULT = re.compile(r"""(?xi)    # Sample matched pattern 2 Adults
                               (\d+)    # Starts with a digit repeats one or more times
                               (?:  # Should not suppose to be a part of matched pattern
                                \s+ # Followed by space repeats zero or more times
                               )
                               (?:  # Should not suppose to be a part of matched pattern
                                adults? # Followed by keyword 'adult' with an optional 's' at the end
                               )
                               """)

PATTERN_CHILD = re.compile(r"""(?xi)  # Sample matched pattern 2 Children
                                (\d+) # Starts with digit repeats one or more times
                                (?: # Should not suppose to be a part of matched pattern
                                    \s+ # Followed by space repeats one or more times
                                )
                                (?: # Should not suppose to be a part of matched pattern
                                    child|children  # Followed by keyword child or children
                                )
                                """)

PATTERN_ADULT2 = re.compile(r"""
                                (?xi)   # Sample matched pattern 2 Adults
                                (\d+)   # Starts with digit repeats one or more times
                                (?: # Should not suppose to be a part of matched pattern
                                    .*  # Capture everything in between
                                )
                                (?: # Should not suppose to be a part of matched pattern
                                    adults?    # till adult or adults occurs in matched pattern
                                )
                                """)
PATTERN_CHILD2 = re.compile(r"""
                            (?xi)   # Sample matched pattern 2 Children
                            (\d+)   # Starts with digit repeats one or more times
                            (?:
                                .*  # Capture everything in between except new line
                            )
                            (?: # Should not suppose to be a part of matched pattern
                                child|children  # till child or children occurs in matched pattern
                            )
                            """)

PATTERNS_CHILDREN = ['Number of Children', 'Number of Children/Per Room']
PATTERNS_ADULTS = ['Number of Adults', 'Number of Adults/Per Room', 'Adult\(s\)', 'Adults']

# Rule for extracting amenities
PATTERN_AMENITIES = re.compile(r"""(?xis)   # Sample matched pattern 1 kind bed no smoking, Free Hi Speed Internet, Iron&Ironing Board,
                                           # In-Room Coffee Maker, Hair Dryer, Free Wireless High Speed Internet Access, In-Room
                                   (?<![A-Za-z]) # Skip matched pattern if any letters small/capital
                                                 # comes before the pattern mentioned in next group
                                   (?: # Shouldn't suppose to be part of matched pattern since we are interested in amenities
                                       # and it is expected that below mentioned keyword do come along with amenities.
                                        king|queen|double|twin|full|non-smoking|nonsmoking|no smoking # Followed by list of keywords mentioned
                                   )*   # Repeats zero or more times
                                   (?: # shouldn't be part of matched pattern
                                        \s|,|:|beds?    # Followed by list of tokens mentioned
                                   )
                                   (.*)""") # Capture everything except new line

# Category for extracting bed_type
CATEGORY_BED = {'king': ['king room', 'king suite'],
                'queen': ['queen room', 'queen suite', 'queen'],
                'double': ['double room', 'double suite']}
# Pattern to extract bed type
PATTERN_BED1 = re.compile(r"""(?x)      # Sample matched pattern 1 kind bed
                              (
                                \d{1,3} # Starts with digit repeats 1 - 3 times
                              )
                              (?: # should'nt be included in matched pattern
                                .*?  # captured everything that comes
                              )
                              (king|queen|double|twin|full) # till any of keyword from the mentioned list occurred
                              """)
PATTERN_BED2 = re.compile(r'(?xi)(?<![A-Za-z])(king|queen|double|twin|full)')
PATTERN_BED3 = re.compile(r"""
                              (?xi)     # Sample matched pattern 1 kind bed
                              (\d{1,3}) # Starts with digit repeats 1 - 3 times
                              (?: # shouldn't be included in matched pattern
                                \s # Followed by a single space
                              )
                              (king|queen|double|twin|full) # Followed by keywords mentioned in list
                              """)


# Pattern to extract star rating
PATTERN_RATING = re.compile(r"""
                                (?xi) # Sample matched pattern: 3 star
                                (\d+) # Starts with digit repeats one or more time
                                (?:   # shouldn't be included in matched pattern
                                    \s?\-?star  # Followed by a space optional and a dash optional and a keyword 'star'
                                )
                                | # OR
                                (?:
                                   star\s?\-?   # keyword 'star' comes before rating value followed by a single space optional
                                                # and a single dash optional
                                )
                                (\d+)   # Followed by a digit repeats one or more times
                                 """)

# Pattern to extract numbers of rooms booked
PATTERN_ROOMS = re.compile(r"""
                                (?xi)   # Sample matched pattern: 1 rooms
                                (\d+)   # Starts with digit repeats one or more time
                                (?:     # shouldn't be included in matched pattern
                                    \s+ # Followed by space repeats one or more times
                                )
                                (?: # shouldn't be included in matched pattern
                                    rooms|room  # Followed by keywords rooms or room
                                )+ # repeats one or more times
                                """)
PATTERNS_ROOMS = ['NUMBER OF ROOMS', 'Number of Rooms', 'Number of rooms', 'Rooms', 'Suites:']

# Pattern to extract hotel url
PATTERN_URL = re.compile(r"""^(?xi)  # Sample matched pattern: Hotel Website
                               \s*  # Starts with a space repeats zero or more times
                               (Hotel)? # Followed by keyword 'Hotel' optional
                               \s*Website\s*$   # Should end with keyword 'Website' enclosed in space that repeats
                                                # zero or more times
                               """)

HOTEL_PATTERNS = {'_guests': ['Clients', 'Guests', 'GUESTS', 'GUESTS PER ROOM', 'Total Occupants'],
                  '_room_type': ['ROOM INFORMATION(?!:)', 'Room Information(?!:)', 'Room Description', 'Room Details:',
                                 'ROOM TYPE', 'Room Type', 'Room\(s\) Booked',
                                 'Room:', r'Room\(s\)']
                  # hmm, how to handle the special case of "date".  Probably move that into the
                  # is_label tester, stripping noise words off the end of the title
                  # '_arrival': ['Arrive', 'Arrival', 'Check-in', 'Check in', 'Check-In', 'Check In'],
                  # '_departure': ['Depart', 'Departure', 'Check-out', 'Check out', 'Check-Out', 'Check Out']
}

# Patterns to find label content of hotel address
HOTEL_ADDRESS_PATTERN = ['Property Address', 'Address']

# Patterns to find label content for guests
PATTERNS_GUESTS = ['Welcome', 'Name', 'Guest Information', 'Guest Name', 'Guest', 'Guest name \(for check\-in\)']

# Patterns to extract hotel name from subject
PATTERNS_NAME = [r'(?xs)(?:Your\s*)?(?:Reservation\s*)(?:Confirmation)(?:.*?)(?:for|at|-)(.*)(?<!\d)$']

# Patterns to extract hotel name from email
HOTEL_NAME_PATTERNS = ['Property name', 'Property Name']

# TODO: We have a plan of converting pattern label text to lower case allowing
# removing duplicates from list, certainly will help to reduce size for list.
# We will be working on it if it is possible to do so.

# Patterns to extract arrival and departure time from email
ARRIVAL_PATTERNS = ['CHECK-IN TIME', 'Check-In Time', 'Check In', 'Check-in Time']
DEPARTURE_PATTERNS = ['CHECK-OUT TIME', 'Check-Out Time', 'Check Out', 'Check-out Time']

# Patterns to extract arrival and departure date from email
PATTERNS_ARRIVAL_DATE = ['YOUR STAY DATES', 'CHECK-IN DATE', 'Check-In Date', 'Check-in', 'Check in', 'Check-In',
                         'Check In', 'Arrive', 'Arrival Date', 'Arrival']
PATTERNS_DEPARTURE_DATE = ['CHECK-OUT DATE', 'Check-out', 'Check out', 'Check-Out', 'Check Out', 'Depart', 'Departure',
                           'Check-Out Date', 'Departure Date']

#  Pattern to determine either time is combine with date or not
PATTERN_TIME_IN_DATE = re.compile(r"""(?xi)     # Sample matched pattern: am
                                       a\.?m\.? # Starts with 'a' followed by an optional dot
                                                # and 'm' and again an optional dot
                                       |    # or
                                       p\.?m\.? # Starts with 'p' followed by an optional dot
                                                # and 'm' and again an optional dot
                                       """)

# Pattern to find if hotel follows refundable policy
PATTERNS_REFUNDABLE = ['Rate and Cancellation Details', 'RATE RULES AND CANCELLATION POLICY',
                       'Rate Rules and Cancellation Policy']

# Pattern to find if hotel is non-smoking
PATTERNS_NONSMOKING = ['non smoking', 'nonsmok', 'non-smoking', 'smoke-free', 'smoke free']

# Tokens that are not suppose to be present in hotel address
TOKENS_ADDRESS = ['T:', 'Tel:', 'F:', 'Fax:', 'Phone:', 'Fax:', 'Front Desk:', 'non-refundable']

# Label Pattern to find amenities
LABEL_AMENITIES = ['Complimentary']

# Following are the keywords over which we decide whether hotel follows non-refundable policy or not
TOKEN_NONREFUNDABLE = {'no charge': False, 'Cancellation penalties may apply': True, 'non-refundable': True}

# Following are the time which we are suppose to use as default time when there is no
# explicit time given for Departure and Arrival
DEFAULT_ARRIVAL_TIME = '4:00 PM'
DEFAULT_DEPARTURE_TIME = '11:00 AM'

# Pattern to deal with Arrival/Departure date separated by dash
PATTERN_DATE = re.compile(r"""
                            (?x)
                            (\d{4}) # Starts with a digit repeats 4 times
                            (?:.*)  # Capture everything except new line till
                            (\d{4}) # we have digits that repeats 4 times
                            """)

# Pattern to look for arrival and departure time.
PATTERN_ARRIVAL_DEPARTURE_TIME = re.compile(r"""^
                                                (?xi)   # Sample matched pattern 3:00 AM
                                                (
                                                    \s*        # Starts with a space repeats zero or more times
                                                    \d{1,2}:   # Followed by digit repeats 1 - 2 times and single semicolon
                                                    \d{2}\s*   # and again digit repeats 2 times and a space repeats zero or more time
                                                )
                                                (AM|PM\s*$) # Should end with keywords AM or PM and space repeats zero or more times
                                                """)

# Pattern to identify if text pattern look like a header text
PATTERN_HEADER_TEXT = re.compile(r"""
                                    (?x)    # Sample matched pattern: Arrival Date:
                                    (.*)    # Capture everything till we
                                    (:)$    # have a semicolon at the end of the string
                                    """)

# Pattern to find hotel description, in future more keywords could be added
# to identify text that represents description of a hotel
PATTERN_HOTEL_DESCRIPTION = re.compile('click here', re.IGNORECASE)

# Look for specific token in date string which won't allow
# date string to be converted in date object
ILLEGAL_TOKENS = ['(', ')', 'at']

# Following strings should not suppose to be present in amenities string
ILLEGAL_STR_AMENITIES = ['preferences', 'reservation', 'cancellations']

# Max value for distance between hotel-name node and phone-number node
DISTANCE = 4

ILLEGAL_HOTEL_TITLE_TOKEN = re.compile("""(?xi)(reserve?|confirm|explore|customer|rewards?)""")

# Following pattern finds out any currency symbol out of email text
PATTERN_CURRENCY_SYMBOL = regex.compile(r'(\p{Sc})', regex.UNICODE)


VALIDATE_PATTERN_PURCHASE_AMOUNT = regex.compile(r"""(?xu)   # Sample matched pattern USD dollars $84.00 USD
    (?:(?<=:))\s*        # Should start with zero or more space
    (\p{Sc})?\s*           # match currency symbol optional followed by a space repeats 0 or more times
    ([A-Z]{3})?\s*         # match character from range [A-Z] and repeat three times
    (([A-z ]*)\s(?i)dollars?)?\s* # match character from range [A-z] or a space  zero or more time followed by a
                                  # space and 'dollars' keyword case insensitive
    (\p{Sc})?\s*           # match currency symbol optional followed by a space repeats 0 or more times
    (([A-z ]*)\s(?i)dollars?)?\s*
    \s*         # Followed a space repeats 0 or more times
    (\d+)       # Followed by digit repeats one or more times
    (,\d{3})*  # Followed by pattern of a comma with 3 digits together repeats 0 or more times
    (?:\.\d{2}) # Followed by pattern of dot and 2 digits together shouldn't be part
                # of matched string

        \s*     # Followed by a space repeats zero or more times
        (
            \(? # Followed by bracket optional, there might be a case when purchase amount
                # is enclosed in brackets therefore making it optional
                ([A-Z]{3}|([A-z ]*)\s(?i)dollars?)  # Capture capital letters of length 3 or letters or a space repeats
                                                    # zero or more time followed by a
                                                    # space and 'dollars' keyword case insensitive
            \)?
        )?  # Currency part sometimes is not under same element could be collected from sibling element,
            # therefore making it as optional
       # Group to capture currency code out of purchase amount
    (\*|\s|\-)* # string should ends with a * or a space repeats zero or more times
                              """)

VALIDATE_PATTERN_PURCHASE_AMOUNT_ONLY = regex.compile(r"""(?xu)\s*(\p{Sc}\s*\d+)\s* """)