"""
Currency converter util, to convert one currency to
another currency at a provided date
"""

import requests
from dateutil import parser
from lola.itinerary_parsing.utils import convert_to_desired_val


def get_url_content(url):
    """
    Hit the url and return its content
    """
    response = requests.get(url)
    response.raise_for_status()
    if response.text == 'False':
        return None
    return response.content


class CurrencyConverter(object):
    """
    Class to create one currency to another currency at
    a certain date. By default USD is used as a default
    base currency
    """

    def __init__(self, source_currency, base_currency="USD"):
        self.base_currency = base_currency
        self.source_currency = source_currency
        self.conversion_api_base = "http://currencies.apps.grandtrunk.net/getrate/%s/%s/%s"

    def get_conversion_rate(self, date):
        if isinstance(date, basestring):
            date = parser.parse(date)
        date_formatted_str = date.strftime("%Y-%m-%d")

        if self.source_currency:
            if self.source_currency.lower() == 'usd':
                return 100
            return self.get_currency_converstion_rate(date_formatted_str)
        return 100

    def get_currency_converstion_rate(self, date):
        """
        This function returns the conversion rate from source
        to base currency at the provided date
        """
        url = self.conversion_api_base % (date, self.source_currency, self.base_currency)
        conversion_rate = get_url_content(url)
        return round(convert_to_desired_val(conversion_rate, float) * 100, 5)
