"""
This file contains itinerary validator class and its methods.
"""
import json

from lola.itinerary_parsing.validation_json_schema import BaseValidator
from lola.itinerary_parsing.validation_json_schema.hotel_validator import HotelValidator
from lola.itinerary_parsing.validation_json_schema.flight_validator import FlightValidator
from lola.itinerary_parsing.validation_json_schema.error_msgs import INVALID_ITINERARY

AIRLINE = 'flight'
HOTEL = 'hotel'


class ItineraryValidator(BaseValidator):
    """
    This class contains hotel and flight itinerary validator methods
    """
    def __init__(self, itinerary):
        self.itinerary = itinerary

    def validate(self):
        """
        Validate flight and hotel itinerary information
        """
        if self.itinerary.get(AIRLINE):
            validator = FlightValidator(self.itinerary.get('flight'))
            return validator.validate()
        elif self.itinerary.get(HOTEL):
            validator = HotelValidator(self.itinerary.get('hotel'))
            return validator.validate()
        else:
            return {'error': True, 'error_msg': json.dumps([INVALID_ITINERARY])}
